/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package tdd_with_fitnesse;

import java.util.logging.Level;
import java.util.logging.Logger;
import static org.hamcrest.Matchers.is;
import org.junit.Test;
import static org.junit.Assert.*;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import org.mockito.junit.MockitoJUnitRunner;

/**
 *
 * @author Selvyn
 */
@RunWith(MockitoJUnitRunner.class)
public class TimeFormatterMockTests
{
    @Mock
    Calculator workCalculator;
    
    //@Rule 
    //public MockitoRule mockitoRule = MockitoJUnit.rule();
    
    @Test
    public void test_calculateHoursWorked_called()
    {
        final TimeSheet itsTimeSheet = new TimeSheet();

        // Arrange
        itsTimeSheet.setCalculator(workCalculator);
        itsTimeSheet.setCheckInTime("9:34AM");
        itsTimeSheet.setCheckOutTime("16:44");
        itsTimeSheet.setHourlyRate("7.38");
        
        // Act
        itsTimeSheet.execute();

        verify(workCalculator).calculateHoursWorked("09:34", "16:44");
    }

    @Test
    public void test_force_results()
    {
        final TimeSheet itsTimeSheet = new TimeSheet();

        // Arrange
        String checkInTime = "9:00 AM";
        String checkInTimeNormalized = "09:00";
        String checkOutTime = "17:00";
        String hourlyRate = "7";
        String expectedPay = "56.00";
        
        // This is not an assertion, the workCalculator.calculateHoursWorked() will
        // return  8.00 when it is called
        when(workCalculator.calculateHoursWorked(checkInTimeNormalized, checkOutTime)).thenReturn(8.00);
        
        // Assign the mock object to the calculator
        itsTimeSheet.setCalculator(workCalculator);
        // Assign the other values
        itsTimeSheet.setCheckInTime(checkInTime);
        itsTimeSheet.setCheckOutTime(checkOutTime);
        itsTimeSheet.setHourlyRate(hourlyRate);

        // Act
        itsTimeSheet.execute();
        
        // Assert
        assertThat( itsTimeSheet.Pay(), is(expectedPay) );
    }
    
    @Test
    public void test_with_a_spy()
    {
        try
        {
            final TimeSheet itsTimeSheet = new TimeSheet();
            TimeSheet theSpy = spy(itsTimeSheet);
            
            // Arrange
            String checkInTime = "9:00 AM";
            String checkOutTime = "17:00";
            String hourlyRate = "7";
            String expectedPay = "56.00";
            
            doReturn(expectedPay).when(theSpy).Pay2();
            
            itsTimeSheet.setCheckInTime(checkInTime);
            itsTimeSheet.setCheckOutTime(checkOutTime);
            itsTimeSheet.setHourlyRate(hourlyRate);
            
            // Act
            itsTimeSheet.execute();
            
            assertThat( itsTimeSheet.Pay(), is(theSpy.Pay2()) );
        } catch (Exception ex)
        {
            Logger.getLogger(TimeFormatterMockTests.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Test
    public void test_method_count()
    {
        final TimeSheet itsTimeSheet = new TimeSheet();

        // Arrange
        String checkInTime = "9:00 AM";
        String checkInTimeNormalized = "09:00";
        String checkOutTime = "17:00";
        String hourlyRate = "7";
        String expectedPay = "56.00";
        
        when(workCalculator.calculateHoursWorked(checkInTimeNormalized, checkOutTime)).thenReturn(8.00);
        
        // Assign the mock object to the calculator
        itsTimeSheet.setCalculator(workCalculator);
        itsTimeSheet.setCheckInTime(checkInTime);
        itsTimeSheet.setCheckOutTime(checkOutTime);
        itsTimeSheet.setHourlyRate(hourlyRate);

        // Act
        itsTimeSheet.execute();
        
        verify(workCalculator, times(1)).calculateHoursWorked(checkInTimeNormalized, checkOutTime);
        assertThat( itsTimeSheet.Pay(), is(expectedPay) );
    }
}
