/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package tdd_with_fitnesse;

import static org.hamcrest.Matchers.equalTo;
import org.junit.Test;
import org.junit.Rule;
import org.junit.rules.ExpectedException;
import static org.junit.Assert.*;

/**
 *
 * @author Selvyn
 */
public class TimeFormatterTest
{
    /**
     * Test of getInstance method, of class TimeFormatter.
     */
    @Test
    public void testGetInstance()
    {
        System.out.println("getInstance");
        TimeFormatter result = TimeFormatter.getInstance();
        assertNotNull(result);
    }

    @Rule
    public ExpectedException thrown = ExpectedException.none();
    // This is a daa member that ca be reused across multiple tests
    
    @Test
    // This approach test the type of the exception thrown and the expected details in the exception
    public void testLengthToShort() throws Exception
    {
        System.out.println("Test length to short");
        // arrange
        thrown.expect(BadlyFormedTime.class);
        thrown.expectMessage(equalTo("Time length too short"));
 
        // act
        String input = "8:2";
        TimeFormatter instance = new TimeFormatter();
        instance.format(input);
    }
    
    @Test
    public void testLengthToLong() throws Exception
    {
        System.out.println("Test length to long");
 
        // act
        String input = "08:25PMM";
        TimeFormatter instance = new TimeFormatter();
        instance.format(input);
    }
    
    /**
     * Test of format method, of class TimeFormatter.
     */
    @Test
    public void testFormat_AM() throws Exception
    {
        System.out.println("Test Format AM");
        String input = "08:25 AM";
        TimeFormatter instance = new TimeFormatter();
        String expResult = "08:25";
        String result = instance.format(input);
        assertEquals(expResult, result);
    }

    @Test(expected=BadlyFormedTime.class)
    // This approach only tests the type of the exception thrown
    public void testFormat_BAD_PM() throws Exception
    {
        System.out.println("Test Format BAD PM");
        String input = "13:25AM";
        TimeFormatter instance = new TimeFormatter();
        String expResult = "13:25";
        String result = instance.format(input);
        assertEquals(expResult, result);
    }

    @Test
    public void testFormat_Edge_1202PM() throws Exception
    {
        System.out.println("format");
        String input = "12:02PM";
        TimeFormatter instance = new TimeFormatter();
        String expResult = "12:02";
        String result = instance.format(input);
        assertEquals(expResult, result);
    }

    @Test
    public void testFormat_Edge_1202AM() throws Exception
    {
        System.out.println("format");
        String input = "12:02AM";
        TimeFormatter instance = new TimeFormatter();
        String expResult = "00:02";
        String result = instance.format(input);
        assertEquals(expResult, result);
    }

    @Test
    public void testFormat_GOOD_PM() throws Exception
    {
        System.out.println("format");
        String input = "13:25PM";
        TimeFormatter instance = new TimeFormatter();
        String expResult = "13:25";
        String result = instance.format(input);
        assertEquals(expResult, result);
    }

    @Test
    public void testFormat_PM_TO_24() throws Exception
    {
        System.out.println("format");
        String input = "4:25PM";
        TimeFormatter instance = new TimeFormatter();
        String expResult = "16:25";
        String result = instance.format(input);
        assertEquals(expResult, result);
    }

    /**
     * Test of tokenise method, of class TimeFormatter.
     */
    @Test
    public void testTokenise_numbers()
    {
        System.out.println("tokenise");
        String input = "09:13";
        String inputTest[] = {"09", "13"};
        TimeFormatter instance = new TimeFormatter();
        instance.tokenise(input, ":");
        assertArrayEquals(inputTest, instance.getTokens());
    }
    
    @Test
    public void testTokenise_numbers_with_PM()
    {
        System.out.println("tokenise");
        String input = "09:13 PM";
        String inputTest[] = {"09", "13", "PM"};
        TimeFormatter instance = new TimeFormatter();
        instance.tokenise(input, ": ");
        assertArrayEquals(inputTest, instance.getTokens());
    }
    
    @Test
    public void testTokenise_numbers_with_AM()
    {
        System.out.println("tokenise");
        String input = "09:13  am";
        String inputTest[] = {"09", "13", "AM"};
        TimeFormatter instance = new TimeFormatter();
        instance.tokenise(input, ": ");
        assertArrayEquals(inputTest, instance.getTokens());
    }

    @Test
    public void testTokeniseSplit_numbers()
    {
        System.out.println("tokenise");
        String input = "09:13";
        String inputTest[] = {"09", "13"};
        TimeFormatter instance = new TimeFormatter();
        String result[] = instance.tokeniseWithSplit(input, ":|\\s+");
        assertArrayEquals(inputTest, result);
    }

    @Test
    public void testTokeniseSplit_numbers_PM()
    {
        System.out.println("tokenise");
        String input = "09:13 PM";
        String inputTest[] = {"09", "13", "PM"};
        TimeFormatter instance = new TimeFormatter();
        String result[] = instance.tokeniseWithSplit(input, ":|\\s+");
        assertArrayEquals(inputTest, result);
    }

    @Test
    public void testTokeniseSplit_24HR_AM_Edge()
    {
        System.out.println("tokenise");
        String input = "13:13 AM";
        String inputTest[] = {"ERR", "ERR", "ERR"};
        TimeFormatter instance = new TimeFormatter();
        String result[] = instance.tokeniseWithSplit(input, ":|\\s+");
        assertArrayEquals(inputTest, result);
    }

    @Test
    public void testTokeniseSplit_numbers_AM()
    {
        System.out.println("tokenise");
        String input = "09:13 AM";
        String inputTest[] = {"09", "13", "AM"};
        TimeFormatter instance = new TimeFormatter();
        String result[] = instance.tokeniseWithSplit(input, ":|\\s+");
        assertArrayEquals(inputTest, result);
    }

    @Test
    public void testTokeniseSplit_numbers_AM_NO_Space()
    {
        System.out.println("tokenise");
        String input = "09:13AM";
        String inputTest[] = {"09", "13", "AM"};
        TimeFormatter instance = new TimeFormatter();
        String result[] = instance.tokeniseWithSplit(input, ":|\\s+");
        assertArrayEquals(inputTest, result);
    }

    @Test
    public void testTokeniseSplit_numbers_PM_NO_Space()
    {
        System.out.println("tokenise");
        String input = "09:13PM";
        String inputTest[] = {"09", "13", "PM"};
        TimeFormatter instance = new TimeFormatter();
        String result[] = instance.tokeniseWithSplit(input, ":|\\s+");
        assertArrayEquals(inputTest, result);
    }

    @Test
    public void testTokeniseSplit_24hr_HMM_to_HHMM()
    {
        System.out.println("tokenise");
        String input = "9:13";
        String inputTest[] = {"9", "13"};
        TimeFormatter instance = new TimeFormatter();
        String result[] = instance.tokeniseWithSplit(input, ":|\\s+");
        assertArrayEquals(inputTest, result);
    }
}
