/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package tdd_with_fitnesse;

import java.text.NumberFormat;
import java.util.Arrays;
import java.util.Collection;
import static org.hamcrest.Matchers.is;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

/**
 *
 * @author Selvyn
 */

@RunWith(Parameterized.class)
public class TimeSheetAcceptanceTest
{
    private String  currentTestId;
    private String  checkInTime;
    private String  checkOutTime;
    private String  checkInTimeNormalized = "NIL";
    private String  checkOutTimeNormalized = "NIL";
    private double  hourlyRate = 0;
    private double  hoursWorked = 0;
    private double  amountToBePaid = 0;
    private TimeSheet itsTimeSheet = new TimeSheet();
    private NumberFormat noFormatter = NumberFormat.getNumberInstance();
            
    public TimeSheetAcceptanceTest( String testId,
                                    String checkIn, String checkOut,
                                    String hourlyRate, 
                                    String checkInNormalized,
                                    String checkOutNormalized,
                                    String hoursWorked, 
                                    String pay )
    {
        noFormatter.setMaximumFractionDigits(2);
        noFormatter.setMinimumFractionDigits(2);

        this.currentTestId = testId;
        this.checkInTime = checkIn;
        this.checkOutTime = checkOut;
        this.checkInTimeNormalized = checkInNormalized;
        this.checkOutTimeNormalized = checkOutNormalized;
        this.hourlyRate = Double.parseDouble(hourlyRate);
        this.hoursWorked = Double.parseDouble(hoursWorked);
        this.amountToBePaid = Double.parseDouble(pay);
    }
    
    @Parameters
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][] { //
                { "test 01", "9:00 AM", "9:00 PM", "7", "09:00", "21:00", "12.00", "84.00" },
                { "test 02", "10:30 AM", "12:30 PM", "4", "10:30", "12:30", "2.00", "8.00" },
                { "test 03", "9:45AM", "12:45 PM", "10", "09:45", "12:45", "3.00", "30.00" },
                { "test 04", "9:30 AM", "12:15", "10", "09:30", "12:15", "2.75", "27.50" },
                { "test 05", "09:30", "12:15 PM", "10", "09:30", "12:15", "2.75", "27.50" },
                { "test 06", "9:00 AM", "15:00", "8", "09:00", "15:00", "6.00", "48.00" },
                { "test 07", "09:00", "3:00 PM", "8", "09:00", "15:00", "6.00", "48.00" },
                { "test 09", "9:05 PM", "4:00 PM", "7", "21:05", "16:00", "0.00", "0.00" },
                { "test 10", "13:05 AM", "4:30 PM", "7", "ERR", "16:30", "0.00", "0.00" },
                { "test 11", "8:59", "09:01", "8", "08:59", "09:01", "0.03", "0.27" }
                });
    }
    
    @Test
    public  void    test_are_time_entries_correct()
    {
        // Arrange
        itsTimeSheet.setCheckInTime(checkInTime);
        itsTimeSheet.setCheckOutTime(checkOutTime);
        itsTimeSheet.setHourlyRate(noFormatter.format(hourlyRate));

        // Act
        itsTimeSheet.execute();

        // Assert
        String actualCheckInTimeNormalized = itsTimeSheet.CheckInTimeNormalized();
        String actualCheckOutTimeNormalized = itsTimeSheet.CheckOutTimeNormalized();
        String actualHoursWorked = itsTimeSheet.HoursWorked();
        String actualPay = itsTimeSheet.Pay();
        String expectedHoursWorked = noFormatter.format(this.hoursWorked);
        String expectedPay = noFormatter.format(this.amountToBePaid);

        assertThat(errorMessage(), actualCheckInTimeNormalized, is(this.checkInTimeNormalized));
        assertThat(errorMessage(), actualCheckOutTimeNormalized, is(this.checkOutTimeNormalized));
        assertThat(errorMessage(), actualHoursWorked, is(expectedHoursWorked));
        assertThat(errorMessage(), actualPay, is(expectedPay));
    }

    public  String  errorMessage()
    {
        String  result = currentTestId;
        
        return currentTestId;
    }
}
